console.log('hello world');

// modules
const { merge, omit, compose } = require("ramda");
const uuid = require("uuid/v4");
var jwk = require("jwk-lite");
var jwk2pem = require("jwk-to-pem");
const kid = uuid()

/*
  generate ES256 Key Pair
  
  A key pair contains a private and public key. Using the ECDSA algorithm the jwk.generateKey function generates the keys and returns them via a promise
*/
const keys = await jwk.generateKey("ES256");

/*
  create json jwk docs

  In order to use these keys, we need to export them to a js object and modify the shape of the object to include some meta data to describe the keys.
*/
const privateJwk = convert(await jwk.exportKey(keys.privateKey))
const publicJwk = convert(await jwk.exportKey(keys.publicKey))

/*
  Using the jwk2pem library we can convert the key to a pem file, which our Jwt module prefers.
  
  You should be able to see the pem in the output below for the privateKey
*/
const pem = jwk2pem(privateJwk, { private: true })

// helper functions
/*
  JWT Libraries (https://jwt.io/#libraries-io) provide sign and verify functionality using JsonWebKeys (JWK)
  
  In order for the JsonWebKeys (JWK) documents to be usuable by the JWT Libraries, convert the returned js object by appending the kid, alg, and use nodes for each key.
*/
function convert (k) {
  const core = { kid, alg: "ES256", use: "sig" };
  const unneededKeys = ["ext", "key_ops"];
  return compose(merge(core), omit(unneededKeys))(k)
}

// Show the private key
pem;


const jwtPayload = {
  iss: "https://from.app.com",
  aud: "https://to.api.com",
  exp: Math.round(Date.now() / 1000 + 300),
  iat: Math.round(Date.now() / 1000),
  jti: uuid(),
  tenant: "mytenant"
}

const jwtHeader = {
  alg: "ES256",
  typ: "JWT",
  kid,
  jku: "https://www.exampleapp.com/.well-known/jwks.json"
}

JSON.stringify(jwtPayload, null, 2);;

const jwt = require('jsonwebtoken')
const token = jwt.sign(jwtPayload, pem, { header: jwtHeader})
;

const express = require('express')
const app = express()

/*
  This endpoint can be included in the jwt header so that the service can locate the public key to validate the token.
*/
app.get('/.well-known/jwks.json', (req, res) => {
  res.send([publicJwk])
})
/*
  This is a simple endpoint to show a link to the jwks public key file
*/
app.get('/', (req, res) => res.send(`
<!doctype html>
<html>
  <head>
    <title>ES256 Token Example</title>
  </head>
  <body>
    <h1>ES256 Token Example</h1>
    <a href="/.well-known/jwks.json">Click here to see jwks.json</a>
  </body>
</html>
`))

app.listen(3000);

`
<div style="height: 200px">
  <a href="https://jwt.io/?value=${token}" target="_blank">Open JWT.IO</a>
</div>
`;
const publicPem = jwk2pem(publicJwk);
